#!/usr/bin/env python
# -*- coding: utf-8 -*-
 
# Importe
import sys
import dbus
import time
import osso
import os
import subprocess
from functools import partial
from PyQt4.QtGui import QApplication, QMainWindow, QFileDialog
from PyQt4.QtCore import Qt
from PyQt4.uic import loadUi
from PyQt4.phonon import Phonon
reload(sys)
sys.setdefaultencoding('utf-8')
 
# Klasse für das Hauptfenster, abgeleitet von QMainWindow und
class N900_Toolbox(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self,parent=None)
        self.ui = loadUi("toolbox.ui")
        self.ui.setAttribute(Qt.WA_Maemo5AutoOrientation, 1)
        self.ui.show()

        self.accel_is_shown = False

        # for vibration and led
        self._MCE_SERVICE = 'com.nokia.mce'
        self._MCE_REQUEST_PATH = '/com/nokia/mce/request'
        self._MCE_REQUEST_IF = 'com.nokia.mce.request'
        self._ENABLE_LED = 'req_led_pattern_activate'
        self._DISABLE_LED = 'req_led_pattern_deactivate'
        self._VIBRATE = 'req_vibrator_pattern_activate'
        self._VIBRATE_PATTERN= 'PatternChatAndEmail'
        self._LED_PATTERN = 'PatternCommunicationCall'
        self.osso_c = osso.Context('program', '1.0', False)
        self.rpc = osso.Rpc(self.osso_c)

        # for playing sounds with phonon
        self.m_media = Phonon.MediaObject(self)
        audioOutput = Phonon.AudioOutput(Phonon.MusicCategory, self)
        Phonon.createPath(self.m_media, audioOutput)
        self.soundfile = ""

        # connects 
        self.ui.pushButton_small_message.clicked.connect(\
                  partial(self.show_notification_dialog, "single"))
        self.ui.pushButton_large_message.clicked.connect(\
                  partial(self.show_notification_dialog, "multiline"))
        self.ui.pushButton_email_like.clicked.connect(self.show_email_like_notification)
        self.ui.pushButton_vibrate.clicked.connect(self.vibrate)
        self.ui.pushButton_led.clicked.connect(self.trigger_led)
        self.ui.pushButton_select_sound.clicked.connect(self.select_sound_file)
        self.ui.pushButton_play_sound.clicked.connect(self.play_sound)
        self.ui.horizontalSlider.valueChanged.connect(self.update_label)
        self.ui.pushButton_accel.clicked.connect(self.show_accel)
        self.ui.pushButton_minimize.clicked.connect(self.minimize)

    def show_notification_dialog(self, mode="single"):
        '''
        provides notification dialog
        '''
        bus = dbus.SystemBus()
        message =  str(self.ui.lineEdit.text())
        iface = dbus.Interface(bus.get_object('org.freedesktop.Notifications',
                                        '/org/freedesktop/Notifications'),
                                        'org.freedesktop.Notifications')
        if mode == "single":
            iface.SystemNoteInfoprint(message)
        elif mode == "multiline":
            iface.SystemNoteDialog(str(message), dbus.UInt32(0), 'Ok')

    def show_email_like_notification(self):
        '''
        provides notification dialog e-mail style
        '''
        bus = dbus.SessionBus()
        message =  str(self.ui.lineEdit.text())
        proxy = bus.get_object('org.freedesktop.Notifications', '/org/freedesktop/Notifications')
        interface = dbus.Interface(proxy,dbus_interface='org.freedesktop.Notifications')
        interface.Notify('Notification', 0, 'control_bluetooth_paired', 'Headline', message, [], {}, 0)

    def vibrate(self):
        '''
        trigger vibration on n900
        '''
        repetation = self.ui.horizontalSlider.value()
        for _ in range(repetation):
            self.rpc.rpc_run(self._MCE_SERVICE, self._MCE_REQUEST_PATH,self._MCE_REQUEST_IF,
                   self._VIBRATE,rpc_args=(self._VIBRATE_PATTERN,"",""),use_system_bus=True)
            time.sleep(1)

    def minimize(self):
        #TODO Use python dbus
        subprocess.Popen(['dbus-send', '--type=signal', '--session', '/com/nokia/hildon_desktop', 'com.nokia.hildon_desktop.exit_app_view'])


    def trigger_led(self):
        self.rpc.rpc_run(self._MCE_SERVICE, self._MCE_REQUEST_PATH,
                         self._MCE_REQUEST_IF,self._ENABLE_LED,
                         rpc_args=(self._LED_PATTERN,"",""),use_system_bus=True)
 
    def select_sound_file(self):
        self.soundfile = QFileDialog.getOpenFileName(self, u'Open File','/home/user/MyDocs/.sounds/')
        if self.soundfile:
            self.ui.pushButton_select_sound.setText(os.path.basename(str(self.soundfile)))
        else:
            self.ui.pushButton_select_sound.setText("select soundfile")

    def play_sound(self):
        # to use volume control, app needs an applicationname (app.setApplicationName)
        if self.soundfile:
           self.m_media.setCurrentSource(Phonon.MediaSource(self.soundfile))
           self.m_media.play()

    def update_label(self, value):
        self.ui.label_slider_value.setText(str(value))

    def show_accel(self):
        #TODO
        reader = AccelReader()
        self.accel_is_shown = not self.accel_is_shown
	while self.accel_is_shown:        
		print reader.get_pos()
 
class AccelReader:
        # found at http://lesloueizeh.com/jdieter/accelreader.py
	def __init__(self):
		bus = dbus.SystemBus()
		self.accel = bus.get_object('com.nokia.mce', '/com/nokia/mce/request', 'com.nokia.mce.request')
		orientation, stand, face, x, y, z = self.accel.get_device_orientation()
		(self.x, self.y, self.z) = (x, y, z)
		self.time = time.time()

	def get_pos(self):	
		orientation, stand, face, x, y, z = self.accel.get_device_orientation()
		new_time = time.time()
		time_modifier = (new_time - self.time)
		self.time = new_time
		time_modifier = time_modifier * 2

		if time_modifier > 1:
			time_modifier = 1
		elif time_modifier < 0.1:
			time_modifier = 0.1

		self.x = self.x + (x - self.x) * time_modifier
		self.y = self.y + (y - self.y) * time_modifier
		self.z = self.z + (z - self.z) * time_modifier
	
		return (self.x, self.y, self.z)

# Starten des Programms
if __name__ == "__main__":
    # Qt-Umgebung erzeugen
    app = QApplication(sys.argv)
    app.setApplicationName('toolbox') # für Phonon, 
    # ein Objekt mit dem Namen fenster aus unserer Klasse
    # Minimalbeispiel erzeugen
    toolbox = N900_Toolbox()
    # Programmstart
    sys.exit(app.exec_())
